@@ -5,10 +5,10 @@
  *  This module implements expression functions that
  *  query and get state information from AutoGen data.
  *
- *  Time-stamp:        "2010-08-06 12:51:19 bkorb"
+ *  Time-stamp:        "2011-02-17 15:33:17 bkorb"
  *
  *  This file is part of AutoGen.
- *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -152,7 +152,7 @@ find_entry_value(SCM op, SCM obj, SCM test)
         }
 
         field  = AG_SCM_STR02SCM(pE->val.pzText);
-        result = gh_call2(op, field, test);
+        result = AG_SCM_APPLY2(op, field, test);
         if (! isIndexed)
             while (result == SCM_BOOL_F) {
 
@@ -161,7 +161,7 @@ find_entry_value(SCM op, SCM obj, SCM test)
                     break;
 
                 field = AG_SCM_STR02SCM(pE->val.pzText);
-                result = gh_call2(op, field, test);
+                result = AG_SCM_APPLY2(op, field, test);
             }
 
         if (OPT_VALUE_TRACE >= TRACE_EXPRESSIONS)
@@ -300,7 +300,8 @@ str2int_ver(char* pz)
 SCM
 ag_scm_version_compare(SCM op, SCM v1, SCM v2)
 {
-    tSCC  zVer[] = "version str";
+    static char const  zVer[] = "version str";
+
     ver_type_t val1 = str2int_ver(ag_scm2zchars(v1, zVer));
     ver_type_t val2 = str2int_ver(ag_scm2zchars(v2, zVer));
     v1 = SCM_FROM(val1);
@@ -323,7 +324,7 @@ ag_scm_version_compare(SCM op, SCM v1, SCM v2)
 SCM
 ag_scm_chdir(SCM dir)
 {
-    tSCC zChdirDir[] = "chdir directory";
+    static char const zChdirDir[] = "chdir directory";
 
     scm_chdir(dir);
 
@@ -711,8 +712,9 @@ ag_scm_suffix(void)
 SCM
 ag_scm_tpl_file(SCM full)
 {
-    if (AG_SCM_BOOL_P(full) && SCM_NFALSEP(full)) {
-        tSCC* sfx[] = { "tpl", NULL };
+    if (AG_SCM_BOOL_P(full) && AG_SCM_NFALSEP(full)) {
+        static char const * const sfx[] = { "tpl", NULL };
+
         char z[AG_PATH_MAX];
         if (SUCCESSFUL(findFile(pzTemplFileName, z, sfx, NULL)))
             return AG_SCM_STR02SCM(z);
@@ -727,20 +729,20 @@ ag_scm_tpl_file(SCM full)
 static SCM
 do_tpl_file_line(int line_delta, char const * fmt)
 {
-    char * buf;
+    void * args[2] = {
+        [0] = (void*)pCurTemplate->pzTplFile,
+        [1] = (void*)((long)pCurMacro->lineNo + line_delta)
+    };
+    char * buf = strrchr(args[0], DIRCH);
+    if (buf != NULL)
+        args[0] = buf + 1;
 
     {
-        size_t sz = strlen(fmt) + strlen(pCurTemplate->pzTplFile) + 24;
+        size_t sz = strlen(fmt) + strlen(args[0]) + 24;
         buf = ag_scribble(sz);
     }
 
-    {
-        void * args[2];
-        args[0] = (void*)pCurTemplate->pzTplFile;
-        args[1] = (void*)((long)pCurMacro->lineNo + line_delta);
-        sprintfv(buf, fmt, (snv_constpointer*)args);
-    }
-
+    sprintfv(buf, fmt, (snv_constpointer*)args);
     return AG_SCM_STR02SCM(buf);
 }
 
@@ -762,7 +764,7 @@ do_tpl_file_line(int line_delta, char const * fmt)
  *  @end example
  *
  *  It is also safe to use the formatting string, "%2$d".  AutoGen uses
- *  an argument vector version of printf: @xref{snprintfv}
+ *  an argument vector version of printf: @xref{snprintfv},
  *  and it does not need to know the types of each argument in order to
  *  skip forward to the second argument.
 =*/
@@ -826,13 +828,10 @@ ag_scm_def_file_line(SCM obj, SCM fmt)
 {
     static char const zFmt[]  = "from %s line %d";
     char const * pzFmt = zFmt;
-    char    zScribble[ 1024 ];
-    char*   pzScrib = zScribble;
-
-    tDefEntry*  pE;
+    char * buf;
     ag_bool     x;
 
-    pE = findDefEntry(ag_scm2zchars(obj, "ag value"), &x);
+    tDefEntry * pE = findDefEntry(ag_scm2zchars(obj, "ag value"), &x);
 
     /*
      *  IF we did not find the entry we are looking for
@@ -845,26 +844,22 @@ ag_scm_def_file_line(SCM obj, SCM fmt)
         pzFmt = ag_scm2zchars(fmt, "file/line format");
 
     {
-        size_t maxlen = strlen(pE->pzSrcFile)
-                      + strlen(pzFmt) + 4 * sizeof(int);
-        if (maxlen >= sizeof(zScribble))
-            pzScrib = (char*)AGALOC(maxlen, "file-line buffer");
+        void * args[2] = {
+            (void*)pE->pzSrcFile,
+            (void*)(long)pE->srcLineNum
+        };
+        size_t maxlen;
+
+        buf = strrchr(args[0], DIRCH);
+        if (buf != NULL)
+            args[0] = buf + 1;
+
+        maxlen = strlen(args[0]) + strlen(pzFmt) + LOG10_2to32 + 1;
+        buf = ag_scribble(maxlen);
+        sprintfv(buf, pzFmt, (snv_constpointer*)args);
     }
 
-    {
-        void* args[2];
-        args[0] = (void*)pE->pzSrcFile;
-        args[1] = (void*)(long)pE->srcLineNum;
-        sprintfv(pzScrib, pzFmt, (snv_constpointer*)args);
-    }
-
-    {
-        SCM res = AG_SCM_STR02SCM(pzScrib);
-        if (pzScrib != zScribble)
-            AGFREE((void*)pzScrib);
-
-        return res;
-    }
+    return AG_SCM_STR02SCM(buf);
 }
 /*
  * Local Variables:
