@@ -1,7 +1,7 @@
 /* -*- buffer-read-only: t -*- vi: set ro:
  *
  * Prototypes for agen5
- * Generated Thu Nov 18 11:54:39 PST 2010
+ * Generated Thu Feb 17 16:59:02 PST 2011
  */
 #ifndef AGEN5_PROTO_H_GUARD
 #define AGEN5_PROTO_H_GUARD 1
@@ -33,15 +33,6 @@ config_dep(tOptions* pOptions, tOptDesc* pOptDesc);
 LOCAL void
 close_server_shell(void);
 
-LOCAL int
-chainOpen(int stdinFd, char const ** ppArgs, pid_t * pChild);
-
-LOCAL pid_t
-openServer(tFdPair* pPair, tCC** ppArgs);
-
-LOCAL pid_t
-openServerFP(tpfPair* pfPair, tCC** ppArgs);
-
 LOCAL char*
 runShell(char const*  pzCmd);
 
@@ -131,7 +122,7 @@ LOCAL void
 print_def(tDefEntry * pDef);
 
 LOCAL tDefEntry*
-findPlace(char* name, tCC* pzIndex);
+findPlace(char* name, char const * pzIndex);
 
 LOCAL void
 readDefines(void);
@@ -143,10 +134,11 @@ unloadDefs(void);
  *  Extracted from expGuile.c
  */
 LOCAL teGuileType
-gh_type_e(SCM typ);
+ag_scm_type_e(SCM typ);
 
 LOCAL SCM
-ag_scm_c_eval_string_from_file_line(tCC* pzExpr, tCC* pzFile, int line);
+ag_scm_c_eval_string_from_file_line(
+    char const * pzExpr, char const * pzFile, int line);
 
 /*
  *  Extracted from expOutput.c
@@ -172,10 +164,10 @@ parseMacroArgs(tTemplate* pT, tMacro* pMac);
 /*
  *  Extracted from funcEval.c
  */
-LOCAL tCC*
+LOCAL char const *
 resolveSCM(SCM s);
 
-LOCAL tCC*
+LOCAL char const *
 evalExpression(ag_bool* pMustFree);
 
 LOCAL SCM
@@ -203,7 +195,7 @@ LOCAL void
 ag_scribble_free(void);
 
 LOCAL char*
-ag_scribble(size_t size);
+ag_scribble(ssize_t size);
 
 /*
  *  Extracted from tpLoad.c
