@@ -1,8 +1,8 @@
 
-;;; Time-stamp:        "2010-02-24 08:42:05 bkorb"
+;;; Time-stamp:        "2011-01-19 11:41:34 bkorb"
 ;;;
 ;;; This file is part of AutoGen.
-;;; AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+;;; AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
 ;;;
 ;;; AutoGen is free software: you can redistribute it and/or modify it
 ;;; under the terms of the GNU General Public License as published by the
@@ -91,15 +91,6 @@
 (define add-cleanup (lambda (t)
    (set! shell-cleanup (string-append shell-cleanup "\n" t "\n"))  ))
 (define tmp-dir "")
-(define make-tmp-dir
-  (lambda () 
-    (begin (if (= tmp-dir "") (set! tmp-dir (shell
-  "mk_tmp_dir ; echo ${tmp_dir}" ))))
-
-  (add-cleanup (string-append
-   "test \"${VERBOSE:-false}\" = true || rm -rf " tmp-dir))
-
-) )
 
 (define header-file     "")
 (define header-guard    "")
@@ -145,10 +136,10 @@
 ;;;  *   emit-string-table}) will insert text and emit the populated table,
 ;;;  *   respectively.
 ;;;  *
-;;;  *   With these functions, it should be much easier to construct structures
-;;;  *   containing string offsets instead of string pointers.  That can be very
-;;;  *   useful when transmitting, storing or sharing data with different address
-;;;  *   spaces.
+;;;  *   With these functions, it should be much easier to construct
+;;;  *   structures containing string offsets instead of string pointers.
+;;;  *   That can be very useful when transmitting, storing or sharing data
+;;;  *   with different address spaces.
 ;;;  *
 ;;;  *   @noindent
 ;;;  *   Here is a brief example copied from the strtable.test test:
@@ -178,17 +169,17 @@
 ;;;  *
 ;;;  *   @noindent
 ;;;  *   I added the @code{(out-push-new)} because the string table text is
-;;;  *   diverted into an output stream named, ``scribble'' and I want to have
-;;;  *   the string table emitted before the string table references.  The string
-;;;  *   table references are also emitted inside the @code{FOR} loop.  So, when
-;;;  *   the loop is done, the current output is suspended under the
-;;;  *   name, ``main'' and the ``scribble'' table is then emitted into the
-;;;  *   primary output.  (@code{emit-string-table} inserts its output directly
-;;;  *   into the current output stream.  It does not need to be the last
-;;;  *   function in an AutoGen macro block.)  Next I @code{ag-fprintf} the
-;;;  *   array-of-pointer declaration directly into the current output.
-;;;  *   Finally I restore the ``main'' output stream and @code{(out-pop #t)}-it
-;;;  *   into the main output stream.
+;;;  *   diverted into an output stream named, ``scribble'' and I want to
+;;;  *   have the string table emitted before the string table references.
+;;;  *   The string table references are also emitted inside the @code{FOR}
+;;;  *   loop.  So, when the loop is done, the current output is suspended
+;;;  *   under the name, ``main'' and the ``scribble'' table is then emitted
+;;;  *   into the primary output.  (@code{emit-string-table} inserts its
+;;;  *   output directly into the current output stream.  It does not need to
+;;;  *   be the last function in an AutoGen macro block.)  Next I
+;;;  *   @code{ag-fprintf} the array-of-pointer declaration directly into the
+;;;  *   current output.  Finally I restore the ``main'' output stream and
+;;;  *   @code{(out-pop #t)}-it into the main output stream.
 ;;;  *
 ;;;  *   Here is the result.  Note that duplicate strings are not repeated
 ;;;  *   in the string table:
@@ -371,4 +362,20 @@
 
 (read-enable 'positions)
 
+;;; /*=gfunc   stack_join
+;;;  *
+;;;  * what:   stack values then join them
+;;;  *
+;;;  * exparg: join , string between each element
+;;;  * exparg: ag-name , name of autogen values to stack
+;;;  *
+;;;  * doc:    This function will collect all the values named @code{ag-name}
+;;;  *         (see the @pxref{SCM stack, stack function}) and join them
+;;;  *         separated by the @code{join} string (see the
+;;;  *         @pxref{SCM join, join function}).
+;;; =*/
+;;;
+(define stack-join (lambda (j-str ag-name)
+  (join j-str (stack ag-name))))
+
 ;;; end of agen5/schemedef.scm
