@@ -1,15 +1,15 @@
 
 /*
- *  Time-stamp:      "2010-04-24 14:17:25 bkorb"
+ *  \file autoopts.h
  *
- *  autoopts.h  $Id: d5e30331d37ca10ec88c592d24d8615dd6c1f0ee $
+ *  Time-stamp:      "2010-12-18 11:53:11 bkorb"
  *
  *  This file defines all the global structures and special values
  *  used in the automated option processing library.
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -110,10 +110,10 @@ typedef int tDirection;
 
 typedef int tSuccess;
 
-#define SUCCEEDED( p )          ((p) == SUCCESS)
-#define SUCCESSFUL( p )         SUCCEEDED( p )
-#define FAILED( p )             ((p) <  SUCCESS)
-#define HADGLITCH( p )          ((p) >  SUCCESS)
+#define SUCCEEDED(p)          ((p) == SUCCESS)
+#define SUCCESSFUL(p)         SUCCEEDED(p)
+#define FAILED(p)             ((p) <  SUCCESS)
+#define HADGLITCH(p)          ((p) >  SUCCESS)
 
 #define ShellAsString(_s)  #_s
 #define DEFAULT_SHELL ShellAsString(CONFIG_SHELL)
@@ -145,7 +145,7 @@ typedef enum {
     OPTION_LOAD_KEEP
 } tOptionLoadMode;
 
-extern tOptionLoadMode option_load_mode;
+static tOptionLoadMode option_load_mode;
 
 /*
  *  The pager state is used by optionPagedUsage() procedure.
@@ -186,9 +186,9 @@ typedef struct {
     { NULL, NULL, OPTST_ ## st, TOPT_UNDEFINED }
 
 #define TEXTTO_TABLE \
-        _TT_( LONGUSAGE ) \
-        _TT_( USAGE ) \
-        _TT_( VERSION )
+        _TT_(LONGUSAGE) \
+        _TT_(USAGE) \
+        _TT_(VERSION)
 #define _TT_(n) \
         TT_ ## n ,
 
@@ -197,41 +197,41 @@ typedef enum { TEXTTO_TABLE COUNT_TT } teTextTo;
 #undef _TT_
 
 typedef struct {
-    tCC*    pzStr;
-    tCC*    pzReq;
-    tCC*    pzNum;
-    tCC*    pzFile;
-    tCC*    pzKey;
-    tCC*    pzKeyL;
-    tCC*    pzBool;
-    tCC*    pzNest;
-    tCC*    pzOpt;
-    tCC*    pzNo;
-    tCC*    pzBrk;
-    tCC*    pzNoF;
-    tCC*    pzSpc;
-    tCC*    pzOptFmt;
-    tCC*    pzTime;
+    char const * pzStr;
+    char const * pzReq;
+    char const * pzNum;
+    char const * pzFile;
+    char const * pzKey;
+    char const * pzKeyL;
+    char const * pzBool;
+    char const * pzNest;
+    char const * pzOpt;
+    char const * pzNo;
+    char const * pzBrk;
+    char const * pzNoF;
+    char const * pzSpc;
+    char const * pzOptFmt;
+    char const * pzTime;
 } arg_types_t;
 
-#define AGALOC( c, w )          ao_malloc((size_t)c)
-#define AGREALOC( p, c, w )     ao_realloc((void*)p, (size_t)c)
-#define AGFREE(_p)              do{void*X=(void*)_p;ao_free(X);}while(0)
-#define AGDUPSTR( p, s, w )     (p = ao_strdup(s))
+#define AGALOC(c, w)          ao_malloc((size_t)c)
+#define AGREALOC(p, c, w)     ao_realloc((void*)p, (size_t)c)
+#define AGFREE(_p)            do{void*X=(void*)_p;ao_free(X);}while(0)
+#define AGDUPSTR(p, s, w)     (p = ao_strdup(s))
 
 static void *
-ao_malloc( size_t sz );
+ao_malloc(size_t sz);
 
 static void *
-ao_realloc( void *p, size_t sz );
+ao_realloc(void *p, size_t sz);
 
 static void
-ao_free( void *p );
+ao_free(void *p);
 
 static char *
-ao_strdup( char const *str );
+ao_strdup(char const *str);
 
-#define TAGMEM( m, t )
+#define TAGMEM(m, t)
 
 /*
  *  DO option handling?
@@ -338,8 +338,8 @@ ao_strdup( char const *str );
 #endif
 
 #ifndef HAVE_STRCHR
-extern char* strchr( char const *s, int c);
-extern char* strrchr( char const *s, int c);
+extern char* strchr(char const *s, int c);
+extern char* strrchr(char const *s, int c);
 #endif
 
 /*
@@ -353,7 +353,8 @@ extern char* strrchr( char const *s, int c);
 /*
  *  File pointer for usage output
  */
-extern FILE* option_usage_fp;
+FILE * option_usage_fp;
+static char const * program_pkgdatadir;
 
 extern tOptProc optionPrintVersion, optionPagedUsage, optionLoadOpt;
 
