@@ -1,13 +1,13 @@
 /**
  * \file configfile.c
  *
- *  Time-stamp:      "2010-07-17 10:25:53 bkorb"
+ *  Time-stamp:      "2011-01-06 16:11:24 bkorb"
  *
  *  configuration/rc/ini file handling.
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -26,59 +26,54 @@
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
+static void
+set_usage_flags(tOptions * opts, char const * flg_txt);
+
 /* = = = START-STATIC-FORWARD = = = */
 static void
-filePreset(
-    tOptions*     pOpts,
-    char const*   pzFileName,
-    int           direction );
+file_preset(tOptions * opts, char const * fname, int dir);
 
 static char*
-handleComment(char* pzText);
+handle_comment(char* pzText);
+
+static char *
+handle_cfg(tOptions * pOpts, tOptState * pOS, char * pzText, int dir);
+
+static char *
+handle_directive(tOptions * pOpts, char * pzText);
+
+static char *
+aoflags_directive(tOptions * pOpts, char * pzText);
+
+static char *
+program_directive(tOptions * pOpts, char * pzText);
+
+static char *
+handle_section(tOptions * pOpts, char * pzText);
+
+static int
+parse_xml_encoding(char ** ppz);
+
+static char *
+trim_xml_text(char * pztxt, char const * pznm, tOptionLoadMode mode);
+
+static void
+cook_xml_text(char * pzData);
+
+static char *
+handle_struct(tOptions * pOpts, tOptState * pOS, char * pzText, int dir);
 
 static char*
-handleConfig(
-    tOptions*     pOpts,
-    tOptState*    pOS,
-    char*         pzText,
-    int           direction );
+parse_keyword(tOptions * pOpts, char * pzText, tOptionValue * pType);
 
 static char*
-handleDirective(
-    tOptions*     pOpts,
-    char*         pzText );
+parse_set_mem(tOptions * pOpts, char * pzText, tOptionValue * pType);
 
-static char*
-handleProgramSection(
-    tOptions*     pOpts,
-    char*         pzText );
+static char *
+parse_value(char * pzText, tOptionValue * pType);
 
-static char*
-handleStructure(
-    tOptions*     pOpts,
-    tOptState*    pOS,
-    char*         pzText,
-    int           direction );
-
-static char*
-parseKeyWordType(
-    tOptions*     pOpts,
-    char*         pzText,
-    tOptionValue* pType );
-
-static char*
-parseSetMemType(
-    tOptions*     pOpts,
-    char*         pzText,
-    tOptionValue* pType );
-
-static char*
-parseValueType(
-    char*         pzText,
-    tOptionValue* pType );
-
-static char*
-skipUnknown(char* pzText);
+static char *
+skip_unkn(char* pzText);
 /* = = = END-STATIC-FORWARD = = = */
 
 
@@ -405,28 +400,24 @@ optionNextValue(tOptionValue const * pOVList,tOptionValue const * pOldOV )
 }
 
 
-/*  filePreset
- *
+/**
  *  Load a file containing presetting information (a configuration file).
  */
 static void
-filePreset(
-    tOptions*     pOpts,
-    char const*   pzFileName,
-    int           direction )
+file_preset(tOptions * opts, char const * fname, int dir)
 {
     tmap_info_t   cfgfile;
     tOptState     optst = OPTSTATE_INITIALIZER(PRESET);
     tAoUL         st_flags = optst.flags;
-    char*         pzFileText =
-        text_mmap(pzFileName, PROT_READ|PROT_WRITE, MAP_PRIVATE, &cfgfile);
+    char *        ftext =
+        text_mmap(fname, PROT_READ|PROT_WRITE, MAP_PRIVATE, &cfgfile);
 
-    if (TEXT_MMAP_FAILED_ADDR(pzFileText))
+    if (TEXT_MMAP_FAILED_ADDR(ftext))
         return;
 
-    if (direction == DIRECTION_CALLED) {
+    if (dir == DIRECTION_CALLED) {
         st_flags = OPTST_DEFINED;
-        direction   = DIRECTION_PROCESS;
+        dir   = DIRECTION_PROCESS;
     }
 
     /*
@@ -435,34 +426,33 @@ filePreset(
      *  If this is called via "optionFileLoad", then the bit is not set
      *  and we consider stuff set herein to be "set" by the client program.
      */
-    if ((pOpts->fOptSet & OPTPROC_PRESETTING) == 0)
+    if ((opts->fOptSet & OPTPROC_PRESETTING) == 0)
         st_flags = OPTST_SET;
 
     do  {
         optst.flags = st_flags;
-        while (IS_WHITESPACE_CHAR(*pzFileText))  pzFileText++;
+        while (IS_WHITESPACE_CHAR(*ftext))  ftext++;
 
-        if (IS_VAR_FIRST_CHAR(*pzFileText)) {
-            pzFileText = handleConfig(pOpts, &optst, pzFileText, direction);
+        if (IS_VAR_FIRST_CHAR(*ftext)) {
+            ftext = handle_cfg(opts, &optst, ftext, dir);
 
-        } else switch (*pzFileText) {
+        } else switch (*ftext) {
         case '<':
-            if (IS_VAR_FIRST_CHAR(pzFileText[1]))
-                pzFileText =
-                    handleStructure(pOpts, &optst, pzFileText, direction);
+            if (IS_VAR_FIRST_CHAR(ftext[1]))
+                ftext = handle_struct(opts, &optst, ftext, dir);
 
-            else switch (pzFileText[1]) {
+            else switch (ftext[1]) {
             case '?':
-                pzFileText = handleDirective(pOpts, pzFileText);
+                ftext = handle_directive(opts, ftext);
                 break;
 
             case '!':
-                pzFileText = handleComment(pzFileText);
+                ftext = handle_comment(ftext);
                 break;
 
             case '/':
-                pzFileText = strchr(pzFileText+2, '>');
-                if (pzFileText++ != NULL)
+                ftext = strchr(ftext + 2, '>');
+                if (ftext++ != NULL)
                     break;
 
             default:
@@ -471,31 +461,30 @@ filePreset(
             break;
 
         case '[':
-            pzFileText = handleProgramSection(pOpts, pzFileText);
+            ftext = handle_section(opts, ftext);
             break;
 
         case '#':
-            pzFileText = strchr(pzFileText+1, '\n');
+            ftext = strchr(ftext + 1, '\n');
             break;
 
         default:
             goto all_done; /* invalid format */
         }
-    } while (pzFileText != NULL);
+    } while (ftext != NULL);
 
- all_done:
+all_done:
     text_munmap(&cfgfile);
 }
 
 
-/*  handleComment
- *
+/**
  *  "pzText" points to a "<!" sequence.
  *  Theoretically, we should ensure that it begins with "<!--",
  *  but actually I don't care that much.  It ends with "-->".
  */
 static char*
-handleComment(char* pzText)
+handle_comment(char* pzText)
 {
     char* pz = strstr(pzText, "-->");
     if (pz != NULL)
@@ -504,19 +493,14 @@ handleComment(char* pzText)
 }
 
 
-/*  handleConfig
- *
+/**
  *  "pzText" points to the start of some value name.
  *  The end of the entry is the end of the line that is not preceded by
  *  a backslash escape character.  The string value is always processed
  *  in "cooked" mode.
  */
-static char*
-handleConfig(
-    tOptions*     pOpts,
-    tOptState*    pOS,
-    char*         pzText,
-    int           direction )
+static char *
+handle_cfg(tOptions * pOpts, tOptState * pOS, char * pzText, int dir)
 {
     char* pzName = pzText++;
     char* pzEnd  = strchr(pzText, '\n');
@@ -529,7 +513,7 @@ handleConfig(
     if (pzText > pzEnd) {
     name_only:
         *pzEnd++ = NUL;
-        loadOptionLine(pOpts, pOS, pzName, direction, OPTION_LOAD_UNCOOKED);
+        loadOptionLine(pOpts, pOS, pzName, dir, OPTION_LOAD_UNCOOKED);
         return pzEnd;
     }
 
@@ -585,67 +569,135 @@ handleConfig(
      *  "pzName" points to what looks like text for one option/configurable.
      *  It is NUL terminated.  Process it.
      */
-    loadOptionLine(pOpts, pOS, pzName, direction, OPTION_LOAD_UNCOOKED);
+    loadOptionLine(pOpts, pOS, pzName, dir, OPTION_LOAD_UNCOOKED);
 
     return pzEnd;
 }
 
 
-/*  handleDirective
- *
+/**
  *  "pzText" points to a "<?" sequence.
- *  For the moment, we only handle "<?program" directives.
+ *  We handle "<?program" and "<?auto-options" directives.
+ *  All others are treated as comments.
  */
-static char*
-handleDirective(
-    tOptions*     pOpts,
-    char*         pzText )
+static char *
+handle_directive(tOptions * pOpts, char * pzText)
 {
-    char   ztitle[32] = "<?";
-    size_t title_len = strlen(zProg);
-    size_t name_len;
+#   define DIRECTIVE_TABLE                      \
+    _dt_(zCfgProg,     program_directive)       \
+    _dt_(zCfgAO_Flags, aoflags_directive)
 
-    if (  (strncmp(pzText+2, zProg, title_len) != 0)
-       || (! IS_WHITESPACE_CHAR(pzText[title_len+2])) )  {
-        pzText = strchr(pzText+2, '>');
-        if (pzText != NULL)
-            pzText++;
-        return pzText;
+    typedef char * (directive_func_t)(tOptions *, char *);
+#   define _dt_(_s, _fn) _fn,
+    static directive_func_t * dir_disp[] = {
+        DIRECTIVE_TABLE
+    };
+#   undef  _dt_
+
+#   define _dt_(_s, _fn) 1 +
+    static int  const   dir_ct  = DIRECTIVE_TABLE 0;
+    static char const * dir_names[DIRECTIVE_TABLE 0];
+#   undef _dt_
+
+    int    ix;
+
+    if (dir_names[0] == NULL) {
+        ix = 0;
+#   define _dt_(_s, _fn) dir_names[ix++] = _s;
+        DIRECTIVE_TABLE;
+#   undef _dt_
     }
 
-    name_len = strlen(pOpts->pzProgName);
-    strcpy(ztitle+2, zProg);
-    title_len += 2;
+    for (ix = 0; ix < dir_ct; ix++) {
+        size_t len = strlen(dir_names[ix]);
+        if (  (strncmp(pzText + 2, dir_names[ix], len) == 0)
+           && (! IS_VALUE_NAME_CHAR(pzText[len+2])) )
+            return dir_disp[ix](pOpts, pzText + len + 2);
+    }
+
+    /*
+     *  We don't know what this is.  Skip it.
+     */
+    pzText = strchr(pzText+2, '>');
+    if (pzText != NULL)
+        pzText++;
+    return pzText;
+}
+
+/**
+ *  handle AutoOpts mode flags
+ */
+static char *
+aoflags_directive(tOptions * pOpts, char * pzText)
+{
+    char * pz = pzText;
+
+    while (IS_WHITESPACE_CHAR(*++pz))  ;
+    pzText = strchr(pz, '>');
+    if (pzText != NULL) {
+
+        size_t len  = pzText - pz;
+        char * ftxt = AGALOC(len + 1, "aoflags");
+
+        memcpy(ftxt, pz, len);
+        ftxt[len] = NUL;
+        set_usage_flags(pOpts, ftxt);
+        AGFREE(ftxt);
+
+        pzText++;
+    }
+
+    return pzText;
+}
+
+/**
+ * handle program segmentation of config file.
+ */
+static char *
+program_directive(tOptions * pOpts, char * pzText)
+{
+    static char const ttlfmt[] = "<?";
+    size_t ttl_len  = sizeof(ttlfmt) + strlen(zCfgProg);
+    char * ttl      = AGALOC(ttl_len, "prog title");
+    size_t name_len = strlen(pOpts->pzProgName);
+
+    memcpy(ttl, ttlfmt, sizeof(ttlfmt) - 1);
+    strcpy(ttl + sizeof(ttlfmt) - 1, zCfgProg);
 
     do  {
-        pzText += title_len;
+        while (IS_WHITESPACE_CHAR(*++pzText))  ;
 
-        if (IS_WHITESPACE_CHAR(*pzText)) {
-            while (IS_WHITESPACE_CHAR(*++pzText))  ;
-            if (  (strneqvcmp(pzText, pOpts->pzProgName, (int)name_len) == 0)
-               && (pzText[name_len] == '>'))  {
-                pzText += name_len + 1;
-                break;
-            }
+        if (  (strneqvcmp(pzText, pOpts->pzProgName, (int)name_len) == 0)
+           && (IS_END_XML_TOKEN_CHAR(pzText[name_len])) ) {
+            pzText += name_len;
+            break;
         }
 
-        pzText = strstr(pzText, ztitle);
+        pzText = strstr(pzText, ttl);
     } while (pzText != NULL);
 
+    AGFREE(ttl);
+    if (pzText != NULL)
+        for (;;) {
+            if (*pzText == NUL) {
+                pzText = NULL;
+                break;
+            }
+            if (*(pzText++) == '>')
+                break;
+        }
+
     return pzText;
 }
 
 
-/*  handleProgramSection
- *
+/**
  *  "pzText" points to a '[' character.
  *  The "traditional" [PROG_NAME] segmentation of the config file.
  *  Do not ever mix with the "<?program prog-name>" variation.
  */
-static char*
-handleProgramSection(
-    tOptions*     pOpts,
-    char*         pzText )
+static char *
+handle_section(tOptions * pOpts, char * pzText)
 {
     size_t len = strlen(pOpts->pzPROGNAME);
     if (   (strncmp(pzText+1, pOpts->pzPROGNAME, len) == 0)
@@ -666,19 +718,169 @@ handleProgramSection(
     return pzText;
 }
 
+/**
+ * parse XML encodings
+ */
+static int
+parse_xml_encoding(char ** ppz)
+{
+#   define XMLTABLE             \
+        _xmlNm_(amp,   '&')     \
+        _xmlNm_(lt,    '<')     \
+        _xmlNm_(gt,    '>')     \
+        _xmlNm_(ff,    '\f')    \
+        _xmlNm_(ht,    '\t')    \
+        _xmlNm_(cr,    '\r')    \
+        _xmlNm_(vt,    '\v')    \
+        _xmlNm_(bel,   '\a')    \
+        _xmlNm_(nl,    '\n')    \
+        _xmlNm_(space, ' ')     \
+        _xmlNm_(quot,  '"')     \
+        _xmlNm_(apos,  '\'')
 
-/*  handleStructure
- *
+    static struct {
+        char const * const  nm_str;
+        unsigned short      nm_len;
+        short               nm_val;
+    } const xml_names[] = {
+#   define _xmlNm_(_n, _v) { #_n ";", sizeof(#_n), _v },
+        XMLTABLE
+#   undef  _xmlNm_
+#   undef XMLTABLE
+    };
+
+    static int const nm_ct = sizeof(xml_names) / sizeof(xml_names[0]);
+    int    base = 10;
+
+    char * pz = *ppz;
+
+    if (*pz == '#') {
+        pz++;
+        goto parse_number;
+    }
+
+    if (IS_DEC_DIGIT_CHAR(*pz)) {
+        unsigned long v;
+
+    parse_number:
+        switch (*pz) {
+        case 'x': case 'X':
+            /*
+             * Some forms specify hex with:  &#xNN;
+             */
+            base = 16;
+            pz++;
+            break;
+
+        case '0':
+            /*
+             *  &#0022; is hex and &#22; is decimal.  Cool.
+             *  Ya gotta love it.
+             */
+            if (pz[1] == '0')
+                base = 16;
+            break;
+        }
+
+        v = strtoul(pz, &pz, base);
+        if ((*pz != ';') || (v > 0x7F))
+            return NUL;
+        *ppz = pz + 1;
+        return (int)v;
+    }
+
+    {
+        int ix = 0;
+        do  {
+            if (strncmp(pz, xml_names[ix].nm_str, xml_names[ix].nm_len)
+                == 0) {
+                *ppz = pz + xml_names[ix].nm_len;
+                return xml_names[ix].nm_val;
+            }
+        } while (++ix < nm_ct);
+    }
+
+    return NUL;
+}
+
+/**
+ * Find the end marker for the named section of XML.
+ * Trim that text there, trimming trailing white space for all modes
+ * except for OPTION_LOAD_UNCOOKED.
+ */
+static char *
+trim_xml_text(char * pztxt, char const * pznm, tOptionLoadMode mode)
+{
+    static char const fmt[] = "</%s>";
+    char   z[64], *pz = z;
+    size_t len = strlen(pznm) + sizeof(fmt) - 2 /* for %s */;
+
+    if (len > sizeof(z))
+        pz = AGALOC(len, "scan name");
+
+    sprintf(pz, fmt, pznm);
+    *pztxt = ' ';
+    pztxt = strstr(pztxt, pz);
+    if (pz != z) AGFREE(pz);
+
+    if (pztxt == NULL)
+        return pztxt;
+
+    if (mode != OPTION_LOAD_UNCOOKED)
+        while (IS_WHITESPACE_CHAR(pztxt[-1]))   len++, pztxt--;
+
+    *pztxt = NUL;
+    return pztxt + len - 1 /* for NUL byte */;
+}
+
+/**
+ */
+static void
+cook_xml_text(char * pzData)
+{
+    char * pzs = pzData;
+    char * pzd = pzData;
+    char   bf[4];
+    bf[2] = NUL;
+
+    for (;;) {
+        int ch = ((int)*(pzs++)) & 0xFF;
+        switch (ch) {
+        case NUL:
+            *pzd = NUL;
+            return;
+
+        case '&':
+            *(pzd++) = \
+                ch = parse_xml_encoding(&pzs);
+            if (ch == NUL)
+                return;
+            break;
+
+        case '%':
+            bf[0] = *(pzs++);
+            bf[1] = *(pzs++);
+            if ((bf[0] == NUL) || (bf[1] == NUL)) {
+                *pzd = NUL;
+                return;
+            }
+
+            ch = strtoul(bf, NULL, 16);
+            /* FALLTHROUGH */
+
+        default:
+            *(pzd++) = ch;
+        }
+    }
+}
+
+/**
  *  "pzText" points to a '<' character, followed by an alpha.
  *  The end of the entry is either the "/>" following the name, or else a
  *  "</name>" string.
  */
-static char*
-handleStructure(
-    tOptions*     pOpts,
-    tOptState*    pOS,
-    char*         pzText,
-    int           direction )
+static char *
+handle_struct(tOptions * pOpts, tOptState * pOS, char * pzText, int dir)
 {
     tOptionLoadMode mode = option_load_mode;
     tOptionValue    valu;
@@ -706,7 +908,7 @@ handleStructure(
             return NULL;
         *pzText = NUL;
         pzText += 2;
-        loadOptionLine(pOpts, pOS, pzName, direction, mode);
+        loadOptionLine(pOpts, pOS, pzName, dir, mode);
         return pzText;
 
     case '>':
@@ -725,28 +927,9 @@ handleStructure(
      */
     *pcNulPoint = NUL;
     pzData = ++pzText;
-
-    /*
-     *  Find the end of the option text and NUL terminate it
-     */
-    {
-        char   z[64], *pz = z;
-        size_t len = strlen(pzName) + 4;
-        if (len > sizeof(z))
-            pz = AGALOC(len, "scan name");
-
-        sprintf(pz, "</%s>", pzName);
-        *pzText = ' ';
-        pzText = strstr(pzText, pz);
-        if (pz != z) AGFREE(pz);
-
-        if (pzText == NULL)
-            return pzText;
-
-        *pzText = NUL;
-
-        pzText += len-1;
-    }
+    pzText = trim_xml_text(pzText, pzName, mode);
+    if (pzText == NULL)
+        return pzText;
 
     /*
      *  Rejoin the name and value for parsing by "loadOptionLine()".
@@ -755,47 +938,24 @@ handleStructure(
     memset(pcNulPoint, ' ', pzData - pcNulPoint);
 
     /*
-     *  If we are getting a "string" value, the process the XML-ish
-     *  %XX hex characters.
+     *  If we are getting a "string" value that is to be cooked,
+     *  then process the XML-ish &xx; XML-ish and %XX hex characters.
      */
-    if (valu.valType == OPARG_TYPE_STRING) {
-        char * pzSrc = pzData;
-        char * pzDst = pzData;
-        char bf[4];
-        bf[2] = NUL;
-
-        for (;;) {
-            int ch = ((int)*(pzSrc++)) & 0xFF;
-            switch (ch) {
-            case NUL: goto string_fixup_done;
-
-            case '%':
-                bf[0] = *(pzSrc++);
-                bf[1] = *(pzSrc++);
-                if ((bf[0] == NUL) || (bf[1] == NUL))
-                    goto string_fixup_done;
-                ch = strtoul(bf, NULL, 16);
-                /* FALLTHROUGH */
-
-            default:
-                *(pzDst++) = ch;
-            }
-        } string_fixup_done:;
-        *pzDst = NUL;
-    }
+    if (  (valu.valType == OPARG_TYPE_STRING)
+       && (mode == OPTION_LOAD_COOKED))
+        cook_xml_text(pzData);
 
     /*
      *  "pzName" points to what looks like text for one option/configurable.
      *  It is NUL terminated.  Process it.
      */
-    loadOptionLine(pOpts, pOS, pzName, direction, mode);
+    loadOptionLine(pOpts, pOS, pzName, dir, mode);
 
     return pzText;
 }
 
 
-/*  internalFileLoad
- *
+/**
  *  Load a configuration file.  This may be invoked either from
  *  scanning the "homerc" list, or from a specific file request.
  *  (see "optionFileLoad()", the implementation for --load-opts)
@@ -877,7 +1037,7 @@ internalFileLoad(tOptions* pOpts)
             strcpy(pz, pOpts->pzRcName);
         }
 
-        filePreset(pOpts, zFileName, inc);
+        file_preset(pOpts, zFileName, inc);
 
         /*
          *  IF we are now to skip config files AND we are presetting,
@@ -935,7 +1095,12 @@ optionFileLoad(tOptions* pOpts, char const* pzProgram)
     if (! SUCCESSFUL(validateOptionsStruct(pOpts, pzProgram)))
         return -1;
 
-    pOpts->pzProgName = pzProgram;
+    {
+        char const ** pp =
+            (char const **)(void *)&(pOpts->pzProgName);
+        *pp = pzProgram;
+    }
+
     internalFileLoad(pOpts);
     return 0;
 }
@@ -986,12 +1151,11 @@ optionLoadOpt(tOptions* pOpts, tOptDesc* pOptDesc)
         /* NOT REACHED */
     }
 
-    filePreset(pOpts, pOptDesc->optArg.argString, DIRECTION_CALLED);
+    file_preset(pOpts, pOptDesc->optArg.argString, DIRECTION_CALLED);
 }
 
 
-/*  parseAttributes
- *
+/**
  *  Parse the various attributes of an XML-styled config file entry
  */
 LOCAL char*
@@ -1019,15 +1183,15 @@ parseAttributes(
 
         switch (find_xat_attribute_id(pzText, len)) {
         case XAT_KWD_TYPE:
-            pzText = parseValueType(pzText+len, pType);
+            pzText = parse_value(pzText+len, pType);
             break;
 
         case XAT_KWD_WORDS:
-            pzText = parseKeyWordType(pOpts, pzText+len, pType);
+            pzText = parse_keyword(pOpts, pzText+len, pType);
             break;
 
         case XAT_KWD_MEMBERS:
-            pzText = parseSetMemType(pOpts, pzText+len, pType);
+            pzText = parse_set_mem(pOpts, pzText+len, pType);
             break;
 
         case XAT_KWD_COOKED:
@@ -1058,7 +1222,7 @@ parseAttributes(
         case XAT_KWD_INVALID:
         invalid_kwd:
             pType->valType = OPARG_TYPE_NONE;
-            return skipUnknown(pzText);
+            return skip_unkn(pzText);
         }
     } while (pzText != NULL);
 
@@ -1066,45 +1230,34 @@ parseAttributes(
 }
 
 
-/*  parseKeyWordType
- *
+/**
  *  "pzText" points to the character after "words=".
  *  What should follow is a name of a keyword (enumeration) list.
  */
 static char*
-parseKeyWordType(
-    tOptions*     pOpts,
-    char*         pzText,
-    tOptionValue* pType )
+parse_keyword(tOptions * pOpts, char * pzText, tOptionValue * pType)
 {
-    return skipUnknown(pzText);
+    return skip_unkn(pzText);
 }
 
 
-/*  parseSetMemType
- *
+/**
  *  "pzText" points to the character after "members="
  *  What should follow is a name of a "set membership".
  *  A collection of bit flags.
  */
 static char*
-parseSetMemType(
-    tOptions*     pOpts,
-    char*         pzText,
-    tOptionValue* pType )
+parse_set_mem(tOptions * pOpts, char * pzText, tOptionValue * pType)
 {
-    return skipUnknown(pzText);
+    return skip_unkn(pzText);
 }
 
 
-/*  parseValueType
- *
+/**
  *  "pzText" points to the character after "type="
  */
-static char*
-parseValueType(
-    char*         pzText,
-    tOptionValue* pType )
+static char *
+parse_value(char * pzText, tOptionValue * pType)
 {
     size_t len = 0;
 
@@ -1117,7 +1270,7 @@ parseValueType(
     if ((len == 0) || (! IS_END_XML_TOKEN_CHAR(*pzText))) {
     woops:
         pType->valType = OPARG_TYPE_NONE;
-        return skipUnknown(pzText);
+        return skip_unkn(pzText);
     }
 
     switch (find_value_type_id(pzText - len, len)) {
@@ -1155,12 +1308,11 @@ parseValueType(
 }
 
 
-/*  skipUnknown
- *
+/**
  *  Skip over some unknown attribute
  */
-static char*
-skipUnknown(char* pzText)
+static char *
+skip_unkn(char* pzText)
 {
     for (;; pzText++) {
         if (IS_END_XML_TOKEN_CHAR(*pzText))  return pzText;
@@ -1169,8 +1321,7 @@ skipUnknown(char* pzText)
 }
 
 
-/*  validateOptionsStruct
- *
+/**
  *  Make sure the option descriptor is there and that we understand it.
  *  This should be called from any user entry point where one needs to
  *  worry about validity.  (Some entry points are free to assume that
@@ -1218,6 +1369,9 @@ validateOptionsStruct(tOptions* pOpts, char const* pzProgram)
         else
             fputs(zAO_Sml, stderr);
 
+        fputs(ShellAsString(AO_CURRENT)  ":"
+              ShellAsString(AO_REVISION) ":"
+              ShellAsString(AO_AGE)      "\n", stderr);
         return FAILURE;
     }
 
@@ -1226,13 +1380,15 @@ validateOptionsStruct(tOptions* pOpts, char const* pzProgram)
      *  and the set of equivalent characters.
      */
     if (pOpts->pzProgName == NULL) {
-        char const* pz = strrchr(pzProgram, DIRCH);
-
+        char const *  pz = strrchr(pzProgram, DIRCH);
+        char const ** pp =
+            (char const **)(void **)&(pOpts->pzProgName);
         if (pz == NULL)
-             pOpts->pzProgName = pzProgram;
-        else pOpts->pzProgName = pz+1;
+             *pp = pzProgram;
+        else *pp = pz+1;
 
-        pOpts->pzProgPath = pzProgram;
+        pp  = (char const **)(void **)&(pOpts->pzProgPath);
+        *pp = pzProgram;
 
         /*
          *  when comparing long names, these are equivalent
