@@ -1,7 +1,8 @@
 
-/*
- *  $Id: 8700c8e91e8094c455392c691d9b6a7d62222240 $
- * Time-stamp:      "2010-02-25 15:59:00 bkorb"
+/**
+ * \file environment.c
+ *
+ * Time-stamp:      "2010-12-06 15:01:45 bkorb"
  *
  *  This file contains all of the routines that must be linked into
  *  an executable to use the generated option processing.  The optional
@@ -10,7 +11,7 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -31,7 +32,7 @@
 
 /* = = = START-STATIC-FORWARD = = = */
 static void
-checkEnvOpt(tOptState * os, char * env_name,
+do_env_opt(tOptState * os, char * env_name,
             tOptions* pOpts, teEnvPresetType type);
 /* = = = END-STATIC-FORWARD = = = */
 
@@ -42,9 +43,9 @@ checkEnvOpt(tOptState * os, char * env_name,
  *  doImmediateOpts and/or doRegularOpts.
  */
 LOCAL void
-doPrognameEnv( tOptions* pOpts, teEnvPresetType type )
+doPrognameEnv(tOptions* pOpts, teEnvPresetType type)
 {
-    char const*   pczOptStr = getenv( pOpts->pzPROGNAME );
+    char const*   pczOptStr = getenv(pOpts->pzPROGNAME);
     token_list_t* pTL;
     int           sv_argc;
     tAoUI         sv_flag;
@@ -60,7 +61,7 @@ doPrognameEnv( tOptions* pOpts, teEnvPresetType type )
      *  Tokenize the string.  If there's nothing of interest, we'll bail
      *  here immediately.
      */
-    pTL = ao_string_tokenize( pczOptStr );
+    pTL = ao_string_tokenize(pczOptStr);
     if (pTL == NULL)
         return;
 
@@ -86,41 +87,41 @@ doPrognameEnv( tOptions* pOpts, teEnvPresetType type )
 
     switch (type) {
     case ENV_IMM:
-        (void)doImmediateOpts( pOpts );
+        (void)doImmediateOpts(pOpts);
         break;
 
     case ENV_ALL:
-        (void)doImmediateOpts( pOpts );
+        (void)doImmediateOpts(pOpts);
         pOpts->curOptIdx = 1;
         pOpts->pzCurOpt  = NULL;
         /* FALLTHROUGH */
 
     case ENV_NON_IMM:
-        (void)doRegularOpts( pOpts );
+        (void)doRegularOpts(pOpts);
     }
 
     /*
      *  Free up the temporary arg vector and restore the original program args.
      */
-    free( pTL );
+    free(pTL);
     pOpts->origArgVect = sv_argv;
     pOpts->origArgCt   = sv_argc;
     pOpts->fOptSet     = sv_flag;
 }
 
 static void
-checkEnvOpt(tOptState * os, char * env_name,
+do_env_opt(tOptState * os, char * env_name,
             tOptions* pOpts, teEnvPresetType type)
 {
-    os->pzOptArg = getenv( env_name );
+    os->pzOptArg = getenv(env_name);
     if (os->pzOptArg == NULL)
         return;
 
-    os->flags    = OPTST_PRESET | OPTST_ALLOC_ARG | os->pOD->fOptState;
-    os->optType  = TOPT_UNDEFINED;
+    os->flags   = OPTST_PRESET | OPTST_ALLOC_ARG | os->pOD->fOptState;
+    os->optType = TOPT_UNDEFINED;
 
     if (  (os->pOD->pz_DisablePfx != NULL)
-       && (streqvcmp( os->pzOptArg, os->pOD->pz_DisablePfx ) == 0)) {
+       && (streqvcmp(os->pzOptArg, os->pOD->pz_DisablePfx) == 0)) {
         os->flags |= OPTST_DISABLED;
         os->pzOptArg = NULL;
     }
@@ -152,21 +153,27 @@ checkEnvOpt(tOptState * os, char * env_name,
      *  The interpretation of the option value depends
      *  on the type of value argument the option takes
      */
-    if (os->pzOptArg != NULL) {
-        if (OPTST_GET_ARGTYPE(os->pOD->fOptState) == OPARG_TYPE_NONE) {
-            os->pzOptArg = NULL;
-        } else if (  (os->pOD->fOptState & OPTST_ARG_OPTIONAL)
-                  && (*os->pzOptArg == NUL)) {
-            os->pzOptArg = NULL;
-        } else if (*os->pzOptArg == NUL) {
-            os->pzOptArg = zNil;
-        } else {
-            AGDUPSTR( os->pzOptArg, os->pzOptArg, "option argument" );
-            os->flags |= OPTST_ALLOC_ARG;
-        }
+    if (OPTST_GET_ARGTYPE(os->pOD->fOptState) == OPARG_TYPE_NONE) {
+        /*
+         *  Ignore any value.
+         */
+        os->pzOptArg = NULL;
+
+    } else if (os->pzOptArg[0] == NUL) {
+        /*
+         * If the argument is the empty string and the argument is
+         * optional, then treat it as if the option was not specified.
+         */
+        if ((os->pOD->fOptState & OPTST_ARG_OPTIONAL) == 0)
+            return;
+        os->pzOptArg = NULL;
+
+    } else {
+        AGDUPSTR(os->pzOptArg, os->pzOptArg, "option argument");
+        os->flags |= OPTST_ALLOC_ARG;
     }
 
-    handleOption( pOpts, os );
+    handle_opt(pOpts, os);
 }
 
 /*
@@ -174,7 +181,7 @@ checkEnvOpt(tOptState * os, char * env_name,
  *  This routine should process in all, immediate or normal modes....
  */
 LOCAL void
-doEnvPresets( tOptions* pOpts, teEnvPresetType type )
+doEnvPresets(tOptions* pOpts, teEnvPresetType type)
 {
     int        ct;
     tOptState  st;
@@ -189,13 +196,13 @@ doEnvPresets( tOptions* pOpts, teEnvPresetType type )
     if ((pOpts->fOptSet & OPTPROC_ENVIRON) == 0)
         return;
 
-    doPrognameEnv( pOpts, type );
+    doPrognameEnv(pOpts, type);
 
     ct  = pOpts->presetOptCt;
     st.pOD = pOpts->pOptDesc;
 
     pzFlagName = zEnvName
-        + snprintf( zEnvName, sizeof( zEnvName ), "%s_", pOpts->pzPROGNAME );
+        + snprintf(zEnvName, sizeof(zEnvName), "%s_", pOpts->pzPROGNAME);
     spaceLeft = AO_NAME_SIZE - (pzFlagName - zEnvName) - 1;
 
     for (;ct-- > 0; st.pOD++) {
@@ -210,14 +217,14 @@ doEnvPresets( tOptions* pOpts, teEnvPresetType type )
          *  IF there is no such environment variable,
          *  THEN skip this entry, too.
          */
-        if (strlen( st.pOD->pz_NAME ) >= spaceLeft)
+        if (strlen(st.pOD->pz_NAME) >= spaceLeft)
             continue;
 
         /*
          *  Set up the option state
          */
-        strcpy( pzFlagName, st.pOD->pz_NAME );
-        checkEnvOpt(&st, zEnvName, pOpts, type);
+        strcpy(pzFlagName, st.pOD->pz_NAME);
+        do_env_opt(&st, zEnvName, pOpts, type);
     }
 
     /*
@@ -229,7 +236,7 @@ doEnvPresets( tOptions* pOpts, teEnvPresetType type )
 
         if (st.pOD->pz_NAME != NULL) {
             strcpy(pzFlagName, st.pOD->pz_NAME);
-            checkEnvOpt(&st, zEnvName, pOpts, type);
+            do_env_opt(&st, zEnvName, pOpts, type);
         }
     }
 }
