@@ -2,11 +2,10 @@
 ##  -*- Mode: shell-script -*-
 ## mklibsrc.sh --   make the libopts tear-off library source tarball
 ##
-## Time-stamp:      "2010-02-24 08:37:19 bkorb"
-##              by: bkorb
+## Time-stamp:      "2011-01-26 14:15:31 bkorb"
 ##
 ##  This file is part of AutoGen.
-##  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+##  AutoGen Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
 ##
 ##  AutoGen is free software: you can redistribute it and/or modify it
 ##  under the terms of the GNU General Public License as published by the
@@ -20,12 +19,8 @@
 ##
 ##  You should have received a copy of the GNU General Public License along
 ##  with this program.  If not, see <http://www.gnu.org/licenses/>.
-## ---------------------------------------------------------------------
-## $Id: 8362d58242318970db5c052f9b99111c837fc0ca $
-## ---------------------------------------------------------------------
-## Code:
 
-set -e -x
+set -e
 
 top_builddir=`cd $top_builddir ; pwd`
 top_srcdir=`cd $top_srcdir ; pwd`
